/*
    SPDX-FileCopyrightText: 2026 Vlad Zahorodnii <vlad.zahorodnii@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#pragma once

namespace KWin
{

class NinePatch
{
public:
    virtual ~NinePatch();

protected:
    NinePatch();
};

} // namespace KWin
